package com.tuoheng.admin.service.road;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tuoheng.admin.entity.*;
import com.tuoheng.admin.enums.RoleEnum;
import com.tuoheng.admin.mapper.*;
import com.tuoheng.admin.query.RoadInformationQuery;
import com.tuoheng.admin.service.road.RoadInformationService;
import com.tuoheng.admin.service.road.query.QueryRoadListByDeptIdService;
import com.tuoheng.admin.utils.CurrentUserUtil;
import com.tuoheng.common.core.common.BaseServiceImpl;
import com.tuoheng.common.core.enums.ServiceExceptionEnum;
import com.tuoheng.common.core.exception.ServiceException;
import com.tuoheng.common.core.utils.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Author ChengWang
 * @Date 2022/11/17
 */
@Service
public class RoadInformationServiceImpl extends BaseServiceImpl<RoadInformationMapper, RoadInformation> implements RoadInformationService {

    @Autowired
    private RoadInformationMapper roadInformationMapper;

    @Autowired
    private SectionMapper sectionMapper;

    @Autowired
    private QueryRoadListByDeptIdService queryRoadListByDeptIdService;

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private DeptMapper deptMapper;

    @Autowired
    private RoadDeptMapper roadDeptMapper;

    @Override
    public JsonResult queryPage(RoadInformationQuery query) {
        if (query.getLimit() == null && query.getPage() == null) {
            return JsonResult.error(ServiceExceptionEnum.PARAMETER_IS_NULL);
        }
        //获取超级管理员登录信息
        String tenantId = CurrentUserUtil.getTenantId();
        if(StringUtils.isEmpty(tenantId)){
            throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
        }
        //获取分页数据
        IPage<RoadInformation> page = new Page<>(query.getPage(), query.getLimit());
        IPage<RoadInformation> pageData = roadInformationMapper.selectPage(page, Wrappers.<RoadInformation>lambdaQuery()
                .eq(RoadInformation::getMark, 1)
                .eq(RoadInformation::getTenantId, tenantId)
                .orderByDesc(RoadInformation::getCreateTime));
        List<RoadInformation> records = pageData.getRecords();
        List<RoadInformation> list = new ArrayList<>();
        for (RoadInformation record : records) {
            //判断公路关联路段信息
            RoadRelation(record);
            list.add(record);
        }
        pageData.setRecords(list);

        return JsonResult.success(pageData);
    }

    @Override
    public JsonResult getListInfo(RoadInformationQuery query) {
        //获取超级管理员登录信息
        String tenantId = CurrentUserUtil.getTenantId();
        if(StringUtils.isEmpty(tenantId)){
            throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
        }
        //当前租户下所有公路信息
        List<RoadInformation> roadInformationList = roadInformationMapper.selectList(new LambdaQueryWrapper<RoadInformation>()
                .eq(RoadInformation::getMark, 1)
                .eq(RoadInformation::getTenantId, tenantId));
        List<RoadInformation> list = new ArrayList<>();
        for (RoadInformation roadInformation : roadInformationList) {
            RoadRelation(roadInformation);
            list.add(roadInformation);
        }

        return JsonResult.success(list);
    }

    /**
     * 根据部门id获取该部门下公路列表
     * @param deptId
     * @return
     */
    @Override
    public JsonResult getListByDeptId(String deptId) {
        return queryRoadListByDeptIdService.getRoadList(deptId);
    }

    @Override
    public JsonResult editInfo(RoadInformation entity) {
        //获取当前超级管理员用户信息
        String userId = CurrentUserUtil.getUserId();
        if(StringUtils.isEmpty(userId)){
            throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
        }
        if(StringUtils.isEmpty(CurrentUserUtil.getTenantId())){
            throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
        }
        if (StringUtils.isEmpty(entity.getId())) {
            //新增
            entity.setCreateUser(userId);
            entity.setCreateTime(DateUtils.now());
            entity.setTenantId(CurrentUserUtil.getTenantId());
        } else {
            //更新
            entity.setUpdateUser(userId);
            entity.setUpdateTime(DateUtils.now());
            entity.setTenantId(CurrentUserUtil.getTenantId());
        }
        super.edit(entity);

        return JsonResult.success();
    }

    /**
     * 根据登录人角色获取该部门下公路信息列表
     * @return
     */
    @Override
    public JsonResult getListByUserType() {
        //获取当前登录人信息  登录用户名
        String userId = CurrentUserUtil.getUserId();
        if(StringUtils.isEmpty(userId)){
            throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
        }
        User user = userMapper.selectOne(Wrappers.<User>lambdaQuery()
                .eq(User::getMark, 1)
                .eq(User::getStatus,1)
                .eq(User::getId, userId));
        if(null == user){
            return JsonResult.error("用户不存在");
        }
        //获取租户id
        String tenantId = user.getTenantId();
        //用户类型角色判断  1超级管理员 2 部门管理员 3普通用户
        if(null == user.getRoleId()){
            throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
        }
        //登录用户若为部门管理员或普通用户需要查询所在部门
        String deptId = user.getDeptId();
        //获取公路列表
        List<RoadInformation> roadInformationList = new ArrayList<>();
        //超级管理员
        if(RoleEnum.SUPER_ADMIN.getCode()== user.getRoleId()){
            roadInformationList = this.getAllRoadInformationList(tenantId);
        }
        //部门管理员
        if(RoleEnum.ADMIN.getCode() == user.getRoleId()){
            //根据当前用户对应的部门id获取该部门及子部门的id集合
            List<String> deptIdList = deptMapper.selectAllChildListById(deptId);
            if(CollectionUtil.isEmpty(deptIdList)){
                return null;
            }
            List<RoadDept> list = roadDeptMapper.selectListByDeptIdList(deptIdList);
            roadInformationList = this.getRoadInformationList(tenantId, list);
        }
        //普通用户
        if(RoleEnum.ORDINARY_USER.getCode() == user.getRoleId()){
            List<RoadDept> roadDeptList = roadDeptMapper.selectList(new LambdaQueryWrapper<RoadDept>()
                    .eq(RoadDept::getTenantId, tenantId)
                    .eq(RoadDept::getDeptId, deptId));
            if (CollectionUtil.isEmpty(roadDeptList)) {
                return null;
            }
            roadInformationList =this.getRoadInformationList(tenantId,roadDeptList);
        }

        return JsonResult.success(roadInformationList);
    }

    /**
     * 根据部门列表获取公路列表
     * @param tenantId
     * @param list
     * @return
     */
    private List<RoadInformation> getRoadInformationList(String tenantId, List<RoadDept> list) {
        List<RoadInformation> roadInformationList;
        List<String> roadIdList = list.stream().map(o -> o.getRoadId()).collect(Collectors.toList());
        Map<String, Object> map = new HashMap<>();
        map.put("tenantId", tenantId);
        map.put("roadIdList", roadIdList);
        map.put("mark",1);
        roadInformationList = roadInformationMapper.getListByMap(map);
        return roadInformationList;
    }

    /**
     * 如果是顶级部门，则查询所有公路列表
     *
     * @param tenantId
     * @return
     */
    private List<RoadInformation> getAllRoadInformationList(String tenantId) {
        List<RoadInformation> roadInformationList = roadInformationMapper.selectList(new LambdaQueryWrapper<RoadInformation>()
                .eq(RoadInformation::getTenantId, tenantId)
                .eq(RoadInformation::getMark, 1));
        if (CollectionUtil.isEmpty(roadInformationList)) {
            return null;
        }
        return roadInformationList;
    }

    /**
     * 判断公路是否关联路段
     *
     * @param roadInformation
     */
    private void RoadRelation(RoadInformation roadInformation) {
        List<Section> sectionList = sectionMapper.selectList(new LambdaQueryWrapper<Section>()
                .eq(Section::getMark, 1)
                .eq(Section::getTenantId, CurrentUserUtil.getTenantId())
                .eq(Section::getRoadId, roadInformation.getId()));
        if (StringUtils.isNotEmpty(sectionList)) {
            //公路绑定路段，关联字段设置为true
            roadInformation.setRelation(true);
        } else {
            //公路未绑定路段，关联字段设置为false
            roadInformation.setRelation(false);
        }
    }
}
