package com.tuoheng.admin.service.user.init;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.tuoheng.admin.constant.SystemConstant;
import com.tuoheng.admin.entity.Role;
import com.tuoheng.admin.entity.Tenant;
import com.tuoheng.admin.entity.User;
import com.tuoheng.admin.mapper.RoleMapper;
import com.tuoheng.admin.mapper.TenantMapper;
import com.tuoheng.admin.mapper.UserMapper;
import com.tuoheng.admin.service.third.oidc.OidcService;
import com.tuoheng.common.core.utils.CommonUtils;
import com.tuoheng.common.core.utils.DateUtils;
import com.tuoheng.common.core.utils.JsonResult;
import com.tuoheng.common.core.utils.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 初始化系统用户业务层处理
 *
 * @author wanjing
 * @team tuoheng
 * @date 2022-12-29
 */
@Slf4j
@Service
public class SystemUserInitService {

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private RoleMapper roleMapper;

    @Autowired
    private TenantMapper tenantMapper;

    @Autowired
    private OidcService oidcService;

    /**
     * 添加用户数据
     *
     * @return
     */
    public JsonResult init(String username) {
        log.info("进入添加用户业务接口");

        Tenant tenant = tenantMapper.selectOne(new LambdaQueryWrapper<Tenant>()
                .eq(Tenant::getMark, 1));
        String tenantId = "";
        if (ObjectUtil.isNotNull(tenant)) {
            tenantId = tenant.getId();
        }

        User user = this.buildUser(username, tenantId);
        JsonResult result = this.check(user, tenantId);
        if (0 != result.getCode()) {
            log.info("初始化系统用户业务接口：校验失败：{}", result.getMsg());
            return result;
        }

        // 添加oidc
        result = oidcService.addUser(user, tenant);
        if (0 != result.getCode()) {
            log.info("初始化系统用户业务接口：添加oidc数据失败：{}", result.getMsg());
            return result;
        }

        Integer rowCount = userMapper.insert(user);
        if (rowCount <= 0) {
            return JsonResult.error(400, "初始化系统用户失败");
        }

        return JsonResult.success();
    }

    /**
     *
     * 构建实体字段数据
     *
     */
    private User buildUser(String username, String tenantId) {
        if (StringUtils.isEmpty(username)) {
            username = "gs";
        }
        User user = new User();
        user.setCode("gs");
        user.setUsername(username);
        user.setRealname("超级管理员");
        user.setNickname("超级管理员");
        user.setCityName("");
        user.setPassword(CommonUtils.password(SystemConstant.INIT_PASSWORD));
        user.setAvatar("imagedir/n1o1j9ndau_1670488066287.png");
        user.setClientId("1,2");
        user.setRoleId(1);
        user.setDeptId("");
        user.setStatus(1);
        user.setTenantId(tenantId);
        user.setCreateTime(DateUtils.now());
        return user;
    }

    /**
     * 检查参数
     *
     * @param user
     * @return
     */
    private JsonResult check(User user, String tenantId) {
        Integer count = userMapper.selectCount(new LambdaQueryWrapper<User>()
                        .eq(User::getTenantId, tenantId)
                        .eq(User::getRoleId, 1)
                        .eq(User::getMark, 1));
        if (count > 0) {
            return JsonResult.error(400, "系统中已经存在超级管理员");
        }

        count = userMapper.selectCount(new LambdaQueryWrapper<User>()
                .eq(User::getTenantId, tenantId)
                .eq(User::getUsername, user.getUsername())
                .eq(User::getMark, 1));
        if (count > 0) {
            return JsonResult.error(400, "系统中已经存在该用户名");
        }
        count = roleMapper.selectCount(new LambdaQueryWrapper<Role>()
                .eq(Role::getTenantId, tenantId)
                .eq(Role::getId, 1)
                .eq(Role::getMark, 1));
        if (count <= 0) {
            return JsonResult.error(400, "该角色不存在");
        }
        return JsonResult.success();
    }

}
